package com.ozacc.springframework.mail.javamail;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.Date;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.BodyPart;
import javax.mail.MessagingException;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimePart;
import javax.mail.internet.MimeUtility;

import org.springframework.mail.javamail.MimeMessageHelper;

/**
 * 
 * @author Tomohiro Otsuka
 * @version $Id: IntlMimeMessageHelper.java,v 1.3 2004/05/03 09:40:25 otsuka Exp $
 */
public class IntlMimeMessageHelper extends MimeMessageHelper {

    protected boolean multipart = false;

    /**
     * 
     * @param mimeMessage
     * @throws MessagingException
     */
    public IntlMimeMessageHelper(MimeMessage mimeMessage) throws MessagingException {
        super(mimeMessage);

        setDate();
    }

    /**
     * 
     * @param mimeMessage
     * @param charset
     * @throws MessagingException
     */
    public IntlMimeMessageHelper(MimeMessage mimeMessage, String encoding)
        throws MessagingException {
        super(mimeMessage, encoding);

        setDate();
    }

    /**
     * @param mimeMessage
     * @param multipart
     * @throws MessagingException 
     */
    public IntlMimeMessageHelper(MimeMessage mimeMessage, boolean multipart)
        throws MessagingException {
        super(mimeMessage, multipart);
        this.multipart = multipart;

        setDate();
    }

    /**
     * @param mimeMessage
     * @param multipart
     * @throws MessagingException 
     */
    public IntlMimeMessageHelper(MimeMessage mimeMessage, boolean multipart, String encoding)
        throws MessagingException {
        super(mimeMessage, multipart, encoding);
        this.multipart = multipart;

        setDate();
    }

    /**
     * 򼨤Dateإåꤷޤ
     * 
     * @throws MessagingException
     */
    protected void setDate() throws MessagingException {
        getMimeMessage().setSentDate(new Date());
    }

    /**
     * ᡼η̾򥻥åȤޤ
     * <p>
     * 󥳡ǥ󥰤JIS(ISO-2022-JP)ξϡ󥳡ǥ
     * बBase64ˤʤ褦ˤƤޤ
     * ϷӤŤMUAȡQuoted-Printable󥳡ǥ󥰤
     * бƤʤ礬뤫Ǥ
     * 
     * @see org.springframework.mail.javamail.MimeMessageHelper#setSubject(java.lang.String)
     */
    public void setSubject(String subject) throws MessagingException {
        if (getEncoding() != null) {
            if (getEncoding().equalsIgnoreCase("iso-2022-jp")) {
                try {
                    getMimeMessage().setSubject(
                        MimeUtility.encodeText(subject, "ISO-2022-JP", "B"));
                } catch (UnsupportedEncodingException e) {
                    throw new MessagingException(e.getMessage());
                }
            } else {
                getMimeMessage().setSubject(subject, getEncoding());
            }
        } else {
            getMimeMessage().setSubject(subject);
        }
    }

    /**
     * Sets the given text directly as content in non-multipart mode
     * respectively as default body part in multipart mode.
     * @param text text to set
     * @param html whether to apply content type "text/html" for an
     * HTML mail, using default content type ("text/plain") else
     */
    public void setText(final String text, boolean html) throws MessagingException {

        // mimeMultipart (ѡ饹ǤprivateСΤᡣ)
        MimeMultipart mimeMultipart = null;
        if (multipart) {
            try {
                mimeMultipart = (MimeMultipart)getMimeMessage().getContent();
            } catch (IOException e) {
                throw new MessagingException(e.getMessage());
            }
        }

        MimePart partToUse = null;
        if (mimeMultipart != null) {
            MimeBodyPart bodyPart = null;

            for (int i = 0; i < mimeMultipart.getCount(); i++) {
                BodyPart bp = mimeMultipart.getBodyPart(i);
                if (bp.getFileName() == null) {
                    bodyPart = (MimeBodyPart)bp;
                }
            }

            if (bodyPart == null) {
                MimeBodyPart mimeBodyPart = new MimeBodyPart();
                mimeMultipart.addBodyPart(mimeBodyPart);
                bodyPart = mimeBodyPart;
            }

            partToUse = bodyPart;
        } else {
            partToUse = getMimeMessage();
        }

        if (html) {
            // need to use a javax.activation.DataSource (!) to set a text
            // with content type "text/html"
            partToUse.setDataHandler(new DataHandler(new DataSource() {
                public InputStream getInputStream() throws IOException {
                    return new ByteArrayInputStream(
                        getEncoding() != null ? text.getBytes(getEncoding()) : text.getBytes());
                }
                public OutputStream getOutputStream() throws IOException {
                    throw new UnsupportedOperationException("Read-only javax.activation.DataSource");
                }
                public String getContentType() {
                    return getEncoding() == null
                        ? "text/html"
                        : "text/html; charset=" + getEncoding();
                }
                public String getName() {
                    return "text";
                }
            }));
        } else {
            if (getEncoding() != null) {
                partToUse.setText(text, getEncoding());
                if (getEncoding().equalsIgnoreCase("iso-2022-jp")) {
                    partToUse.setHeader("Content-Transfer-Encoding", "7bit");
                }
            } else {
                partToUse.setText(text);
            }
        }
    }

}
